//
// Copyright (c) 2022, MindFusion LLC - Bulgaria.
//

using System;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Windows.Forms;

using MindFusion.Drawing;

using SolidBrush = MindFusion.Drawing.SolidBrush;
using Pen = MindFusion.Drawing.Pen;


namespace MindFusion.Diagramming.WinForms.Samples.CS.Genealogy
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();
		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			dbConn.Open();

			// get database data
			dbPersonAdapter.Fill(personSet);
			dbChartAdapter.Fill(chartSet);

			// if there is a chart saved, the program has been run at leat once
			if (chartSet.Tables["FlowChart"].Rows.Count > 0)
			{
				// read the chart
				diagram.LoadFromString(chartSet.FlowChart[0]["FlowChart"] as string);

				// get photos
				foreach (PersonDataSet.PersonRow pr
					in personSet.Tables["Person"].Rows)
				{
					// if there is a photo
					if (!pr.IsPhotoNull())
					{
						ShapeNode b = GetNodeByID(pr.ID);

						if (b != null)
						{
							// get it form the BLOB
							MemoryStream ms = new MemoryStream(pr.Photo);
							Person p = (Person)b.Tag;
							p.Photo = Image.FromStream(ms);

							b.Tag = p;
							b.Image = p.Photo;
						}
					}
				}
			}
		}

		private ShapeNode GetNodeByID(int id)
		{
			foreach (ShapeNode b in diagram.Nodes)
			{
				Person p = (Person)b.Tag;
				if (id == p.Id) return b;
			}

			return null;
		}

		private void AfterBoxCreated(ShapeNode b)
		{
			Person p = new Person();
			p.BirthDate = new DateTime(1980, 1, 1);
			p.Name = "unnamed";
			p.MaleOrFemale = true;

			// display properties dialog
			PersonPropsForm propDlg = new PersonPropsForm(p);
			if (propDlg.ShowDialog(this) == DialogResult.OK)
			{
				// update the person structure
				p.BirthDate = propDlg.BirthDate;
				p.Name = propDlg.PersonName;
				p.Photo = propDlg.Photo;
				p.MaleOrFemale = propDlg.MaleOrFemale;

				// update box image
				b.Image = p.Photo;
				b.ImageAlign = ImageAlign.Fit;

				// update box text
				b.Text = p.Name;
				StringFormat tf = new StringFormat();
				tf.LineAlignment = StringAlignment.Far;
				tf.Alignment = StringAlignment.Center;
				b.TextFormat = tf;

				// update color
				if (p.MaleOrFemale)
				{
                    b.Pen = new Pen(Color.FromArgb(156, 170, 198), 0);
                    b.Brush = new SolidBrush(Color.FromArgb(224, 233, 233));
				}
				else
				{
					b.Pen = new Pen(Color.HotPink, 0);
					b.Brush = new SolidBrush(Color.Pink);
				}
			}

			// create BLOB from the photo
			byte[] buffer = null;
			if (p.Photo != null)
			{
				MemoryStream ms = new MemoryStream();
				p.Photo.Save(ms, ImageFormat.Jpeg);
				buffer = ms.GetBuffer();
			}

			// insert a record
			PersonDataSet.PersonRow pr =
				personSet.Person.AddPersonRow(
					p.BirthDate, p.MaleOrFemale, p.Name, 0, 0, buffer);
			dbPersonAdapter.Update(personSet);

			// save the DB record ID
			p.Id = pr.ID;

			// save the Person structure as box' tag
			b.Tag = p;
		}

		private void MainForm_Closed(object sender, EventArgs e)
		{
			dbChartAdapter.DeleteCommand.ExecuteNonQuery();

			// we dont want to save pictures here
			// they are already in the database Person records
			foreach (ShapeNode b in diagram.Nodes)
			{
				b.Image = null;

				Person p = (Person)b.Tag;
				p.Photo = null;

				b.Tag = p;
			}

			// save the whole chart
			dbChartAdapter.InsertCommand.Parameters.Clear();
			dbChartAdapter.InsertCommand.Parameters.AddWithValue("FlowChart",
				diagram.SaveToString(SaveToStringFormat.Base64, true));
			dbChartAdapter.InsertCommand.ExecuteNonQuery();

			dbConn.Close();
		}

		private void dbPersonAdapter_RowUpdated(object sender, OleDbRowUpdatedEventArgs e)
		{
			if (e.StatementType == StatementType.Insert)
			{
				OleDbCommand idCMD = new OleDbCommand("select @@IDENTITY", dbConn);

				int newID = (int)idCMD.ExecuteScalar();
				e.Row["ID"] = newID;
			}
		}

		private void MenuItem1_Click(object sender, EventArgs e)
		{
			Person p = (Person)pointedBox.Tag;

			// display properties dialog
			PersonPropsForm propDlg = new PersonPropsForm(p);
			if (propDlg.ShowDialog(this) == DialogResult.OK)
			{
				// update the person structure
				p.BirthDate = propDlg.BirthDate;
				p.Name = propDlg.PersonName;
				p.Photo = propDlg.Photo;
				p.MaleOrFemale = propDlg.MaleOrFemale;

				// update box image
				pointedBox.Image = p.Photo;

				// update box text
				pointedBox.Text = p.Name;

				// update color
				if (p.MaleOrFemale)
				{
                    pointedBox.Pen = new Pen(Color.FromArgb(156, 170, 198), 0);
                    pointedBox.Brush = new SolidBrush(Color.FromArgb(224, 233, 233));
				}
				else
				{
					pointedBox.Pen = new Pen(Color.HotPink, 0);
					pointedBox.Brush = new SolidBrush(Color.Pink);
				}
			}

			// create BLOB from the photo
			byte[] buffer = null;
			if (p.Photo != null)
			{
				MemoryStream ms = new MemoryStream();
				p.Photo.Save(ms, ImageFormat.Jpeg);
				buffer = ms.GetBuffer();
			}

			// update record
			PersonDataSet.PersonRow pr =
				personSet.Person.FindByID(p.Id);
			pr.BeginEdit();
			pr.BirthDate = p.BirthDate;
			pr.Name = p.Name;
			pr.Male = p.MaleOrFemale;
			if (buffer == null)
				pr.SetPhotoNull();
			else
				pr.Photo = buffer;

			pr.EndEdit();
			dbPersonAdapter.Update(personSet);

			// save the Person structure as box' tag
			pointedBox.Tag = p;
			pointedBox = null;
		}

		private void MenuItem2_Click(object sender, EventArgs e)
		{
			// Transfer selected box along with its closest relatives to Local view
			ShapeNode b;
			ShapeNode bb;

			localDiagram.ClearAll();
			b = CopyNodeLocal(pointedBox);

			foreach (DiagramLink a in pointedBox.OutgoingLinks)
			{
				bb = CopyNodeLocal(a.Destination as ShapeNode);
				CopyLinkLocal(a, b, bb);
			}

			foreach (DiagramLink a in pointedBox.IncomingLinks)
			{
				bb = CopyNodeLocal(a.Origin as ShapeNode);
				CopyLinkLocal(a, bb, b);
			}

			view.BringIntoView(pointedBox);
			localDiagram.ResizeToFitItems(10, true);
			localView.BringIntoView(b);
		}

		private void diagram_NodeCreated(object sender, NodeEventArgs e)
		{
			AfterBoxCreated(e.Node as ShapeNode);
			localDiagram.ClearAll();
		}

		private void diagram_NodeClicked(object sender, NodeEventArgs e)
		{
			if (e.MouseButton == MouseButton.Right)
			{
				pointedBox = e.Node as ShapeNode;
				cmBox.Show(this, PointToClient(MousePosition));
			}
		}

		private void diagram_LinkCreated(object sender, LinkEventArgs e)
		{
			RelationPropsForm relProps = new RelationPropsForm();
			relProps.ShowDialog();

			if (relProps.ParentToChild)
			{
                e.Link.Pen = new Pen(Color.FromArgb(0, 52, 102), 0);
                e.Link.HeadBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(156, 170, 198));
                e.Link.HeadPen = new Pen(Color.FromArgb(0, 52, 102), 0);
				e.Link.HeadShape = ArrowHeads.PointerArrow;
			}
			else
			{
				e.Link.Pen = new Pen(Color.FromArgb(206, 0, 0), 0);
				e.Link.HeadShape = ArrowHeads.None;
			}

			localDiagram.ClearAll();
		}

		private void diagram_LinkDeleting(object sender, LinkValidationEventArgs e)
		{
			e.Cancel = true;
		}

		private void diagram_NodeDeleting(object sender, NodeValidationEventArgs e)
		{
			e.Cancel = true;
		}

		private void diagram_NodeModified(object sender, NodeEventArgs e)
		{
			localDiagram.ClearAll();
		}

		private void localDiagram_LinkCreated(object sender, LinkEventArgs e)
		{
			RelationPropsForm relProps = new RelationPropsForm();
			relProps.ShowDialog();

            if (relProps.ParentToChild)
            {
                e.Link.Pen = new Pen(Color.FromArgb(0, 52, 102), 0);
                e.Link.HeadBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(156, 170, 198));
                e.Link.HeadPen = new Pen(Color.FromArgb(0, 52, 102), 0);
                e.Link.HeadShape = ArrowHeads.PointerArrow;
            }
            else
            {
                e.Link.Pen = new Pen(Color.FromArgb(206, 0, 0), 0);
                e.Link.HeadShape = ArrowHeads.None;
            }

            //if (relProps.ParentToChild)
            //{
            //    e.Link.Pen = new Pen(Color.Green, 0);
            //    e.Link.HeadShape = ArrowHeads.PointerArrow;
            //}
            //else
            //{
            //    e.Link.Pen = new Pen(Color.DarkGoldenrod, 0);
            //    e.Link.HeadShape = ArrowHeads.None;
            //}

			// Duplicate the arrow in the global view
			ShapeNode dest;
			ShapeNode src;
			dest = diagram.FindNode(e.Link.Destination.Tag) as ShapeNode;
			src = diagram.FindNode(e.Link.Origin.Tag) as ShapeNode;

			DiagramLink a;
			a = diagram.Factory.CreateDiagramLink(src, dest);
			a.Pen = e.Link.Pen;
			a.HeadShape = e.Link.HeadShape;
		}

		private void localDiagram_LinkDeleting(object sender, LinkValidationEventArgs e)
		{
			e.Cancel = true;
		}

		private void localDiagram_NodeCreated(object sender, NodeEventArgs e)
		{
			// Create the box in the global view
			ShapeNode b;

			b = diagram.Factory.CreateShapeNode(e.Node.Bounds);

			// Initialize and save the box
			AfterBoxCreated(b);

			// Destroy the box created and copy from the global view
			localDiagram.Nodes.Remove(e.Node);
			CopyNodeLocal(b);
		}

		private void localDiagram_NodeDeleting(object sender, NodeValidationEventArgs e)
		{
			e.Cancel = true;
		}

		private void localDiagram_NodeModified(object sender, NodeEventArgs e)
		{
			// Adjust modification to global view
			DiagramNode b;

			b = diagram.FindNode(e.Node.Tag);
			b.Bounds = e.Node.Bounds;
		}

		private ShapeNode CopyNodeLocal(ShapeNode b)
		{
			ShapeNode node;

			node = localDiagram.Factory.CreateShapeNode(b.Bounds);
			node.Text = b.Text;
			node.TextFormat = b.TextFormat;
			node.Pen = b.Pen;
			node.Brush = b.Brush;
			node.Tag = b.Tag;
			node.Image = b.Image;
			node.ImageAlign = b.ImageAlign;

			return node;
		}

		private DiagramLink CopyLinkLocal(DiagramLink a, ShapeNode s, ShapeNode d)
		{
			DiagramLink link;

			link = localDiagram.Factory.CreateDiagramLink(s, d);
			link.HeadShape = a.HeadShape;
			link.Pen = a.Pen;
			link.Brush = a.Brush;

			return link;
		}


		private ShapeNode pointedBox = null;
	}
}